use argh::FromArgs;

#[derive(FromArgs, PartialEq, Debug)]
/// Everything you need to selfhost a pronouns.today server
pub struct PronounsTodayArgs {
	#[argh(subcommand)]
	pub command: SubCommand
}

#[derive(FromArgs, PartialEq, Debug)]
#[argh(subcommand)]
pub enum SubCommand {
	Run(Run),
	#[cfg(feature = "embed_static_assets")]
	DumpStatics(DumpStatics),
}

#[derive(FromArgs, PartialEq, Debug)]
#[cfg(feature = "embed_static_assets")]
#[argh(subcommand, name = "dump-statics")]
/// Dump necessary static files to a directory to be served by a webserver.  Serving
/// static files with a dedicated webserver isn't necessary, but may improve performance.
/// If serving static files, all files should be available under the /static/filename.ext
/// route.
pub struct DumpStatics {
	#[argh(option, default = "String::from(\"./static/\")")]
	/// the directory to write the static files to
	///
	/// Defaults to ./static/
	pub destination: String,
}

#[derive(FromArgs, PartialEq, Debug)]
#[argh(subcommand, name = "run")]
/// Run the server from the specified config file.  Commandline options override config
/// options.
pub struct Run {
	#[cfg_attr(feature = "docker",
			   argh(
				   option,
				   default = "String::from(\"/data/config.yml\")",
				   short = 'c'))]
	#[cfg_attr(not(feature = "docker"),
			   argh(
				   option,
				   default = "String::from(\"/etc/pronouns_today.yml\")",
				   short = 'c'))]
	/// path to the config file (generated if needed)
	pub config: String,

	#[argh(switch)]
	/// don't generate a config file if not present
	pub no_generate_cfg: bool,

	#[argh(option, short = 'p')]
	/// the port to listen on
	pub port: Option<u16>,

	#[argh(option)]
	/// default pronoun probabilites (formatted as a prefstring, like the ones in the
	/// used in the custom URLs, e.g. "acaqeawdym")
	pub default_prefstr: Option<String>,
}
